class Stow::Local
  attr_reader :base
  
  def initialize(options = {})
    @base = options[:base]
    raise ArgumentError, 'Must supply base dir' unless @base
  end
  
  def put(file, options = {})
    path = options[:path]
    raise ArgumentError, 'Must supply path' unless path
    instance = File.new(self, path)
    instance.body = file
    instance.save
    instance
  end
  
  def get(path)
    file = File.new(self, path)
    return file if ::File.file?(::File.expand_path(file.path, base))
    raise Stow::NotFoundError, "Unknown file '#{path}'"
  end
  
  class File
    attr_reader :store, :path
    
    def initialize(store, path)
      @store = store
      @path = path
      return if absolute.start_with?(store.base) 
      raise OutsideBaseError, "Path '#{absolute}' leads outside of base '#{store.base}'"
    end
    
    def body=(content)
      @body = content
      @type = if content.respond_to?(:path) then :file
      elsif content.respond_to?(:read) then :io
      else :string
      end
    end

    def body
      case @type
      when :string
        @body
      when :io
        @type = :string
        @body = @body.read
      when :file
        @type = :string
        @body = ::File.open(@body, 'rb') { |file| file.read }
      end
    end

    def save
      FileUtils.mkdir_p(::File.dirname(absolute))
      case @type
      when :string then ::File.open(absolute, 'wb') { |f| f.write(@body) }
      when :file then FileUtils.copy_file(@body.path, absolute)
      when :io then ::File.open(absolute, 'wb') { |f| f.write(@body.read) }
      else raise TypeError, "Can't write #{@body.inspect}" unless persisted?
      end
    end
    
    def copy(options = {})
      path = options[:path]
      store = options[:store] || self.store
      raise ArgumentError, 'Must supply path' unless path
      copy = store.class::File.new(store, path)
      ::File.open(absolute, 'rb') do |file|
        copy.body = file
        copy.save
      end
      copy
    end
    
    def update(attributes)
      self.body = attributes[:body]
      save
    end
    
    def destroy
      return unless persisted?
      FileUtils.rm absolute
      dir = ::File.dirname(absolute)
      Dir.rmdir(dir) if (Dir.entries(dir) - %w(. ..)).empty?
    end
    
    def size
      case @type
      when :string, :io then @body.size
      when :file then ::File.size(@body.path)
      else persisted? ? ::File.size(absolute) : 0
      end
    end
    
    private
    
    def persisted?
      ::File.file?(absolute)
    end
    
    def absolute
      @absolute ||= ::File.expand_path(path, store.base)
    end
  end
  
  class OutsideBaseError < StandardError
  end
end